package org.apithefire.server;

import java.io.IOException;
import java.net.ServerSocket;

public abstract class Connector {
	
	private static final Integer DEFAULT_HTTP_PORT = 80;
	
	protected Integer port;
	
	public Integer getPort() {
		return port;
	}
	
	public void setPort(Integer port) {
		if (port != null && port.intValue() < 1) {
			throw new IllegalArgumentException("Port must be greater than 0.");
		}
		this.port = port;
	}
	
	public void setDefaultPort() {
		this.port = DEFAULT_HTTP_PORT;
	}
	
	public void setUnusedPort() {
		ServerSocket socket = null;
		try {
			socket = new ServerSocket(0);
			this.port = socket.getLocalPort();
		} catch (IOException e) {
			throw new ServerException(e);
		} finally {
			if (socket != null) {
				try {
					socket.close();
				} catch (IOException e) {
					throw new ServerException(e);
				}
			}
		}
	}
	
	protected void validate() {
		if (port == null) {
			throw new ServerException("Connector port is not set.");
		}
	}
	
	protected abstract void configure(Object serverImpl);

}
