package org.apithefire.server;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

class RequestInfoServlet extends HttpServlet {

	@Override
	protected void doGet(HttpServletRequest request,
			HttpServletResponse response) throws ServletException,
			IOException {
		
		response.setContentType("text/html");
		printHtml(response.getWriter(), request);
		response.flushBuffer();
	}

	@Override
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException,
			IOException {
		doGet(request, response);
	}
	
	@SuppressWarnings("unchecked")
	private void printHtml(PrintWriter out, HttpServletRequest request) {
		out.println("<html>");
		out.print("<head><title>");
		out.print("HTTP Request Information");
		out.println("</title></head>");
		out.println("<body>");
		out.println("<h1>HTTP Request Information</h1>");
		out.println("<table border=\"0\">");
		printRow(out, "Protocol", request.getProtocol());
		printRow(out, "Scheme", request.getScheme());
		printRow(out, "Secure", request.isSecure());
		printRow(out, "Server Name", request.getServerName());
		printRow(out, "Server Port", request.getServerPort());
		printRow(out, "Context Path", request.getContextPath());
		printRow(out, "Request URI", request.getRequestURI());
		printRow(out, "Request URL", request.getRequestURL());
		printRow(out, "Path Info", request.getPathInfo());
		printRow(out, "Path Translated", request.getPathTranslated());
		printRow(out, "Character Encoding", request.getCharacterEncoding());
		printRow(out, "Locale", request.getLocale());
		printRow(out, "Locales", toList(request.getLocales()));
		printRow(out, "Authentication Type", request.getAuthType());
		printRow(out, "Remote Host", request.getRemoteHost());
		printRow(out, "Remote Address", request.getRemoteAddr());
		printRow(out, "Remote Port", request.getRemotePort());
		printRow(out, "Remote User", request.getRemoteUser());
		out.println("</table>");
		printHeaders(out, request);
		printParameters(out, request);
		out.println("</body>");
		out.println("</html>");
	}
	
	private void printRow(PrintWriter out, String name, Object value) {
		out.println("<tr>");
		out.print("<td><b>");
		out.print(name);
		out.print(": </b></td>");
		out.print("<td>");
		printValue(out, value);
		out.print("</td>");
		out.println("</tr>");
	}

	@SuppressWarnings("unchecked")
	private void printValue(PrintWriter out, Object value) {
		if (value == null) {
			out.print("<i>&lt;null&gt;</i>");
		} else {
			if (value instanceof Collection) {
				Collection<Object> values = (Collection<Object>) value;
				
				if (values.isEmpty()) {
					out.print("<i>&lt;empty&gt;</i>");
				} else if (values.size() == 1) {
					printValue(out, values.iterator().next());
				} else {
					out.println("<ul>");
					
					for (Object valueItem : values) {
						out.print("<li>");
						printValue(out, valueItem);
						out.print("</li>");
					}
					out.println("</ul>");
				}
			} else if (value.getClass().isArray()) {
				printValue(out, toList((Object[]) value));
			} else if (value instanceof Map) {
				//Map<Object, Object>
			} else {
				out.print(value);
			}
		}
	}
	
	@SuppressWarnings("unchecked")
	private void printHeaders(PrintWriter out, HttpServletRequest request) {
		out.println("<h2>Headers</h2>");
		out.println("<table border=\"1\">");
		out.println("<tr><th>Name</th><th>Value(s)</th></tr>");
		
		Enumeration<String> nameEnm = request.getHeaderNames();
		
		if (nameEnm.hasMoreElements()) {
			while (nameEnm.hasMoreElements()) {
				String name = nameEnm.nextElement();
				List values = toList(request.getHeaders(name));
				
				out.print("<tr><td>");
				out.print(name);
				out.print("</td><td>");
				printValue(out, values);
				out.println("</td></tr>");
			}
		} else {
			out.println("<tr><td colspan=\"2\"><i>&lt;empty&gt;</i></td></tr>");
		}
		out.println("</table>");
	}
	
	@SuppressWarnings("unchecked")
	private void printParameters(PrintWriter out, HttpServletRequest request) {
		out.println("<h2>Parameters</h2>");
		out.println("<table border=\"1\">");
		out.println("<tr><th>Name</th><th>Value(s)</th></tr>");
		
		Map<String, Object> map = request.getParameterMap();
		
		if (map.isEmpty()) {
			out.println("<tr><td colspan=\"2\"><i>&lt;empty&gt;</i></td></tr>");
		} else {
			for (Map.Entry<String, Object> entry : map.entrySet()) {
				printRow(out, entry.getKey(), entry.getValue());
			}
		}
		out.println("</table>");
	}
	
	private List<Object> toList(Enumeration<Object> enm) {
		if (!enm.hasMoreElements()) {
			return Collections.emptyList();
		}
		
		List<Object> list = new ArrayList<Object>();
		
		while (enm.hasMoreElements()) {
			list.add(enm.nextElement());
		}
		return list;
	}
	
	private List<Object> toList(Object[] array) {
		if (array.length <= 0) {
			return Collections.emptyList();
		}
		
		List<Object> list = new ArrayList<Object>();
		
		for (Object item : array) {
			list.add(item);
		}
		return list;
	}
	
}
